@@ -26,10 +26,61 @@ from orchestcli import utils
 if t.TYPE_CHECKING:
     from multiprocessing.pool import AsyncResult
 
+
+def echo(*args, **kwargs) -> None:
+    """Wrapped `click.echo`.
+
+    Note:
+        Will do nothing in case the current CLI command is invoked with
+        the `--json` flag.
+
+    """
+    click_ctx = click.get_current_context(silent=True)
+
+    if click_ctx is None:
+        return click.echo(*args, **kwargs)
+
+    json_flag = click_ctx.params.get("json_flag")
+    if json_flag and json_flag is not None:
+        return
+    else:
+        return click.echo(*args, **kwargs)
+
+
+JECHO_CALLS = 0
+
+
+def jecho(*args, **kwargs) -> None:
+    """JSON echo."""
+    # Invoking `jecho` multiple times within one CLI invocation would
+    # mean that the final output is not JSON parsable.
+    global JECHO_CALLS
+    assert JECHO_CALLS == 0, "`jecho` should only be called once per CLI invocation."
+    JECHO_CALLS += 1
+
+    message = kwargs.get("message")
+    if message is not None:
+        kwargs["message"] = json.dumps(message, sort_keys=True, indent=True)
+    else:
+        if args and args[0] is not None:
+            args = (json.dumps(args[0], sort_keys=True, indent=True), *args[1:])
+    return click.echo(*args, **kwargs)  # type: ignore
+
+
 try:
-    config.load_kube_config()
-except config.config_exception.ConfigException:
-    config.load_incluster_config()
+    try:
+        config.load_kube_config()
+    except config.config_exception.ConfigException:
+        config.load_incluster_config()
+except Exception as e:
+    echo(
+        "Aborting..."
+        f"\nCould not load kube-config file: {e}"
+        "\nFor example for minikube, you need to make sure your cluster"
+        " is started.",
+        err=True,
+    )
+    sys.exit(1)
 
 ORCHEST_NAMESPACE = re.compile("(namespace: )([-a-z]+)")
 
@@ -79,46 +130,6 @@ class LogLevel(str, enum.Enum):
     ERROR = "ERROR"
 
 
-def echo(*args, **kwargs) -> None:
-    """Wrapped `click.echo`.
-
-    Note:
-        Will do nothing in case the current CLI command is invoked with
-        the `--json` flag.
-
-    """
-    click_ctx = click.get_current_context(silent=True)
-
-    if click_ctx is None:
-        return click.echo(*args, **kwargs)
-
-    json_flag = click_ctx.params.get("json_flag")
-    if json_flag and json_flag is not None:
-        return
-    else:
-        return click.echo(*args, **kwargs)
-
-
-JECHO_CALLS = 0
-
-
-def jecho(*args, **kwargs) -> None:
-    """JSON echo."""
-    # Invoking `jecho` multiple times within one CLI invocation would
-    # mean that the final output is not JSON parsable.
-    global JECHO_CALLS
-    assert JECHO_CALLS == 0, "`jecho` should only be called once per CLI invocation."
-    JECHO_CALLS += 1
-
-    message = kwargs.get("message")
-    if message is not None:
-        kwargs["message"] = json.dumps(message, sort_keys=True, indent=True)
-    else:
-        if args and args[0] is not None:
-            args = (json.dumps(args[0], sort_keys=True, indent=True), *args[1:])
-    return click.echo(*args, **kwargs)  # type: ignore
-
-
 class CRObjectNotFound(Exception):
     """CR Object defining Orchest Cluster not found."""
 
@@ -146,7 +157,9 @@ class ClusterStatus(enum.Enum):
     DELETING = "Deleting"
 
 
-def install(cloud: bool, dev_mode: bool, fqdn: t.Optional[str], **kwargs) -> None:
+def install(
+    cloud: bool, dev_mode: bool, no_argo: bool, fqdn: t.Optional[str], **kwargs
+) -> None:
     """Installs Orchest."""
     ns, cluster_name = kwargs["namespace"], kwargs["cluster_name"]
 
@@ -156,13 +169,13 @@ def install(cloud: bool, dev_mode: bool, fqdn: t.Optional[str], **kwargs) -> Non
         if e.reason == "Conflict":
             echo(f"Installing into existing namespace: {ns}.")
 
+    manifest_file_name = "orchest-controller.yaml"
+
     echo("Installing the Orchest Controller to manage the Orchest Cluster...")
     if dev_mode:
         # NOTE: orchest-cli commands to be invoked in Orchest directory
         # root for relative path to work.
-        with open(
-            "services/orchest-controller/deploy/k8s/orchest-controller.yaml"
-        ) as f:
+        with open(f"services/orchest-controller/deploy/k8s/{manifest_file_name}") as f:
             txt_deploy_controller = f.read()
     else:
         version = _fetch_latest_available_version(curr_version=None, is_cloud=cloud)
@@ -175,7 +188,9 @@ def install(cloud: bool, dev_mode: bool, fqdn: t.Optional[str], **kwargs) -> Non
             )
             sys.exit(1)
         try:
-            txt_deploy_controller = _fetch_orchest_controller_manifests(version)
+            txt_deploy_controller = _fetch_orchest_controller_manifests(
+                version, manifest_file_name
+            )
         except RuntimeError as e:
             echo(f"{e}", err=True)
             sys.exit(1)
@@ -185,38 +200,29 @@ def install(cloud: bool, dev_mode: bool, fqdn: t.Optional[str], **kwargs) -> Non
 
     # Deploy the `orchest-controller` and the resources it needs.
     yml_deploy_controller = yaml.safe_load_all(txt_deploy_controller)
-    try:
-        utils.create_from_yaml(
-            k8s_client=API_CLIENT,
-            yaml_objects=yml_deploy_controller,
-        )
-    except utils.FailToCreateError as e:
-        conflicting_resources = []
-        for exc in e.api_exceptions:
-            if exc.status == 409:  # conflict/already exists
-                conflicting_resources.append(json.loads(exc.body)["details"])
-            elif exc.status == 500:
-                exc_msg = json.loads(exc.body)["message"]
-                echo(
-                    f"Installation aborted. Kubernetes API message:\n{exc_msg}",
-                    err=True,
-                )
-                echo(
-                    "\n'orchest install' was probably called too quickly after"
-                    " creation of the Kubernetes cluster. To recover installing"
-                    " Orchest, run:"
-                    "\n\torchest uninstall"
-                    "\nThen wait a few seconds before again running:"
-                    "\n\torchest install",
-                    err=True,
-                )
+    failed_to_create_k8s_objs = []
+    conflicting_k8s_resources = []
+    for yml_document in yml_deploy_controller:
+        if yml_document is None:
+            continue
+        try:
+            utils.create_from_dict(
+                k8s_client=API_CLIENT,
+                data=yml_document,
+            )
+        except utils.FailToCreateError as e:
+            for exc in e.api_exceptions:
+                if exc.status == 409:  # conflict/already exists
+                    conflicting_k8s_resources.append(json.loads(exc.body)["details"])
+                elif exc.status == 500:
+                    failed_to_create_k8s_objs.append(yml_document)
+                else:
+                    raise e
 
-                sys.exit(1)
-            else:
-                raise e
-
-        conflicting_resources_msg = "\n".join(
-            [json.dumps(resource) for resource in conflicting_resources]
+    # Don't overwrite existing/conflicting resources, fail instead.
+    if conflicting_k8s_resources:
+        conflicting_k8s_resources_msg = "\n".join(
+            [json.dumps(resource) for resource in conflicting_k8s_resources]
         )
 
         echo(
@@ -235,11 +241,78 @@ def install(cloud: bool, dev_mode: bool, fqdn: t.Optional[str], **kwargs) -> Non
         echo(
             "If uninstalling doesn't work, then please try to remove the"
             " following resources manually before installing again:"
-            f"\n{conflicting_resources_msg}",
+            f"\n{conflicting_k8s_resources_msg}",
             err=True,
         )
         sys.exit(1)
 
+    # Retry to create k8s objects that could not be created the first
+    # time.
+    if failed_to_create_k8s_objs:
+        retries = 0
+        get_backoff_period = lambda x: 5 * 2**x  # noqa
+        # NOTE: Total possible wait of `sum(5 * 2**x for x in range(4))`
+        # which is 75s.
+        while retries < 4:
+            echo(
+                "Failed to install the Orchest Controller, retrying in"
+                f" {get_backoff_period(retries)} seconds...",
+            )
+            time.sleep(get_backoff_period(retries))
+
+            tmp_failed_to_create_k8s_objs = []
+            for yml_document in failed_to_create_k8s_objs:
+                try:
+                    # NOTE: Use replace instead of create to handle
+                    # "List" kinds.
+                    utils.replace_from_dict(
+                        k8s_client=API_CLIENT,
+                        data=yml_document,
+                    )
+                except utils.FailToCreateError as e:
+                    for exc in e.api_exceptions:
+                        if exc.status == 500:
+                            tmp_failed_to_create_k8s_objs.append(yml_document)
+                            exc_msg = json.loads(exc.body)["message"]
+                        else:
+                            raise e
+
+            failed_to_create_k8s_objs = tmp_failed_to_create_k8s_objs
+            if failed_to_create_k8s_objs:
+                retries += 1
+            else:
+                break
+        else:
+            echo(
+                f"Installation aborted. Kubernetes API message:\n{exc_msg}",
+                err=True,
+            )
+            sys.exit(1)
+
+    # Creating the OrchestCluster custom resource.
+    applications = [{"name": "docker-registry", "config": {}}]
+    if no_argo:
+        echo(
+            "Disabling 'Argo Workflows' installation."
+            "\n\tMake sure 'Argo Workflows' is already installed in your cluster"
+            "\n\tand has the right permissions set in order to work properly"
+            "\n\twith Orchest."
+        )
+    else:
+        applications.append(
+            {
+                "name": "argo-workflow",
+                "config": {
+                    "helm": {
+                        # `singleNamespace` so that Argo acts as a
+                        # namespace level component and only schedule
+                        # workflows in it's own namespace.
+                        "parameters": [{"name": "singleNamespace", "value": "true"}]
+                    }
+                },
+            }
+        )
+
     custom_object = {
         "apiVersion": "orchest.io/v1alpha1",
         "kind": "OrchestCluster",
@@ -248,6 +321,7 @@ def install(cloud: bool, dev_mode: bool, fqdn: t.Optional[str], **kwargs) -> Non
             "namespace": ns,
         },
         "spec": {
+            "applications": applications,
             "orchest": {
                 "orchestHost": fqdn,
                 "orchestWebServer": {"env": [{"name": "CLOUD", "value": str(cloud)}]},
@@ -469,17 +543,19 @@ def update(
         )
         sys.exit(1)
 
+    manifest_file_name = "orchest-controller.yaml"
+
     echo("Updating the Orchest Controller deployment requirements...")
     if dev_mode:
         # NOTE: orchest-cli commands to be invoked in Orchest directory
         # root for relative path to work.
-        with open(
-            "services/orchest-controller/deploy/k8s/orchest-controller.yaml"
-        ) as f:
+        with open(f"services/orchest-controller/deploy/k8s/{manifest_file_name}") as f:
             txt_deploy_controller = f.read()
     else:
         try:
-            txt_deploy_controller = _fetch_orchest_controller_manifests(version)
+            txt_deploy_controller = _fetch_orchest_controller_manifests(
+                version, manifest_file_name
+            )
         except RuntimeError as e:
             echo(f"{e}", err=True)
             sys.exit(1)
@@ -494,10 +570,10 @@ def update(
             k8s_client=API_CLIENT,
             yaml_objects=yml_deploy_controller,
         )
-    except utils.FailToCreateError:
+    except utils.FailToCreateError as e:
         echo(
             "Failed to update Orchest. Could not apply 'orchest-controller'"
-            " manifests.",
+            f" manifests: \n{e}",
             err=True,
         )
         sys.exit(1)
@@ -752,13 +828,16 @@ def patch(
     echo("Successfully patched the Orchest Cluster.")
 
 
-def version(json_flag: bool, **kwargs) -> None:
-    """Gets running Orchest version."""
+def version(json_flag: bool, latest_flag: bool, **kwargs) -> None:
+    """Gets Orchest version."""
     try:
-        version = _get_orchest_cluster_version(
-            kwargs["namespace"],
-            kwargs["cluster_name"],
-        )
+        if latest_flag:
+            version = _fetch_latest_available_version(curr_version=None, is_cloud=False)
+        else:
+            version = _get_orchest_cluster_version(
+                kwargs["namespace"],
+                kwargs["cluster_name"],
+            )
 
     except CRObjectNotFound as e:
         if json_flag:
@@ -1344,10 +1423,12 @@ def _fetch_latest_available_version(
         return None
 
 
-def _fetch_orchest_controller_manifests(version: t.Optional[str]) -> str:
+def _fetch_orchest_controller_manifests(
+    version: t.Optional[str], manifest_file_name: str
+) -> str:
     url = (
         "https://github.com/orchest/orchest"
-        f"/releases/download/{version}/orchest-controller.yaml"
+        f"/releases/download/{version}/{manifest_file_name}"
     )
     resp = requests.get(url, timeout=10)
     if resp.status_code != 200:
